package com.agilefaqs.rft.suggestions;

import static com.agilefaqs.rft.suggestions.RestrictedWords.RestrictedDomainNames;
import static com.agilefaqs.rft.suggestions.RestrictedWords.RestrictedUserNames;

import com.agilefaqs.rft.domain.DomainNameService;

public final class Email {
    private static final int MIN_TOKEN_LENGTH = 3;
    private static final String COM = ".com";
    private static final String AT = "@";
    private static final int MAX_ATTEMPTS = 5;
    private final String userName;
    private final String domainName;
    private final DomainNameService dns;

    private Email(final String userName, final String domainName, final DomainNameService dns) {
        if (isEmpty(userName) || isEmpty(domainName))
            throw new IllegalStateException("User Name and Domain Name cannot be empty");
        this.userName = userName;
        this.domainName = domainName;
        this.dns = dns;
    }

    public static String create(final String userName, final String domainName, final DomainNameService dns) {
        try {
            return new Email(userName, domainName, dns).buildIdWithoutRestrictedWordsAndCelebrityNames();
        } catch (IllegalStateException e) {
            return "";
        }
    }

    private String buildIdWithoutRestrictedWordsAndCelebrityNames() {
        Email current = this;
        if (isCelebrityName())
            current = trimLastCharacter();
        return buildIdWithoutRestrictedWordsAndCelebrityNames(current, 1);
    }

    private String buildIdWithoutRestrictedWordsAndCelebrityNames(final Email last, final int count) {
        if (count == MAX_ATTEMPTS)
            throw new IllegalStateException("Exceeded the Max number of tries");
        String userName = findClosestNonRestrictiveWord(last.userName, RestrictedUserNames, 0);
        String domainName = findClosestNonRestrictiveWord(last.domainName, RestrictedDomainNames, 0);
        Email id = new Email(userName, domainName, dns);
        if (!id.isCelebrityName())
            return id.asString();
        return buildIdWithoutRestrictedWordsAndCelebrityNames(id.trimLastCharacter(), count + 1);
    }

    private String asString() {
        return userName.toLowerCase() + AT + domainName.toLowerCase() + COM;
    }

    private boolean isEmpty(final String token) {
        return token == null || token.length() == 0;
    }

    private Email trimLastCharacter() {
        return new Email(trimLastCharacter(userName), trimLastCharacter(domainName), dns);
    }

    private boolean isCelebrityName() {
        return dns.isCelebrityName(userName, domainName);
    }

    private String findClosestNonRestrictiveWord(final String word, final RestrictedWords restrictedWords, final int count) {
        if (count == MAX_ATTEMPTS)
            throw new IllegalStateException("Exceeded the Max number of tries");
        if (word.length() <= MIN_TOKEN_LENGTH)
            throw new IllegalStateException(word + " is smaller than " + MIN_TOKEN_LENGTH + " chars");
        if (!restrictedWords.contains(word, dns))
            return word;
        return findClosestNonRestrictiveWord(trimLastCharacter(word), restrictedWords, count + 1);
    }

    private String trimLastCharacter(final String word) {
        return word.substring(0, word.length() - 1);
    }
}
