package com.agilefaqs.rft.suggestions;

import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Mockito.mock;

import java.util.List;

import org.junit.Test;

import com.agilefaqs.rft.domain.DomainNameService;
import com.agilefaqs.rft.domain.RandomNumberGenerator;
import com.agilefaqs.rft.domain.UserService;

public class EmailSuggesterTest {
    private int randomCount = 1;
    private final UserService userService = mock(UserService.class);
    private final DomainNameService dns = mock(DomainNameService.class);
    private final RandomNumberGenerator randomNumberGenerator = new RandomNumberGenerator() {
        @Override
        public String next() {
            if (randomCount == 6)
                randomCount = 1;
            return "_random" + randomCount++;
        }
    };

    private final User naresh_from_mumbai = new User("naresh", "jains", "mumbai", "india", "indian");
    private final Context lets = new Context(userService, dns);
    private final EmailSuggester suggester = new EmailSuggester(userService, dns, randomNumberGenerator);

    @Test
    public void suggestIdsUsingNameLocationAndNationality() {
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh@jains.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void avoidRestrictedWordsInIds() {
        lets.assume("naresh").isARestrictedUserName();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("nares@jains.com", "nares@india.com", "nares@indian.com", "nares@mumbai.com");
    }

    @Test
    public void avoidCelebrityNamesInGeneratedIds() {
        lets.assume("naresh", "jains").isACelebrityName();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("nares@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void appendCurrentYearWithFirstNameIfIdIsNotAvailable() {
        lets.assume().identity("naresh@jains.com").isNotAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh2009@jains.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void appendRandomNumberWithFirstNameIfIdIsNotAvailable() {
        lets.assume().identity("naresh@jains.com", "naresh2009@jains.com").isNotAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh_random1@jains.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void tryFiveRandomNumberForFirstIdIfNoIdsAreAvailableUsingFirstNameAndLastName() {
        lets.assume().identity("naresh@jains.com", "naresh2009@jains.com", "naresh_random1@jains.com", "naresh_random2@jains.com",
                "naresh_random3@jains.com", "naresh_random4@jains.com").isNotAvailable();
        lets.assume().identity("naresh_random5@jains.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com").isAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh_random5@jains.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void skipFirstIdIfNoIdsAreAvailableUsingFirstNameAndLastName() {
        lets.assume().identity("naresh@jains.com", "naresh2009@jains.com", "naresh_random1@jains.com", "naresh_random2@jains.com",
                "naresh_random3@jains.com", "naresh_random4@jains.com", "naresh_random5@jains.com").isNotAvailable();
        lets.assume().identity("naresh@india.com", "naresh@indian.com", "naresh@mumbai.com").isAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void skipSecondIdIfNoIdsAreAvailableWithCountryCombination() {
        lets.assume().identity("naresh@india.com", "nareshjains@india.com", "nareshj@india.com", "naresh2009@india.com",
                "nareshj2009@india.com", "naresh_random1@india.com", "naresh_random2@india.com", "naresh_random3@india.com",
                "naresh_random4@india.com", "naresh_random5@india.com").isNotAvailable();
        lets.assume().identity("naresh@jains.com", "naresh@indian.com", "naresh@mumbai.com").isAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh@jains.com", "naresh@indian.com", "naresh@mumbai.com");
    }

    @Test
    public void skipThirdIdIfNoIdsAreAvailableWithCountryMonikerCombination() {
        lets.assume().identity("naresh@indian.com", "nareshjains@indian.com", "nareshj@indian.com", "naresh2009@indian.com",
                "nareshj2009@indian.com", "naresh_random1@indian.com", "naresh_random2@indian.com", "naresh_random3@indian.com",
                "naresh_random4@indian.com", "naresh_random5@indian.com").isNotAvailable();
        lets.assume().identity("naresh@jains.com", "naresh@india.com", "naresh@mumbai.com").isAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh@jains.com", "naresh@india.com", "naresh@mumbai.com");
    }

    @Test
    public void skipForthIdIfNoIdsAreAvailableWithCityCombination() {
        lets.assume().identity("naresh@mumbai.com", "nareshjains@mumbai.com", "nareshj@mumbai.com", "naresh2009@mumbai.com",
                "nareshj2009@mumbai.com", "naresh_random1@mumbai.com", "naresh_random2@mumbai.com", "naresh_random3@mumbai.com",
                "naresh_random4@mumbai.com", "naresh_random5@mumbai.com").isNotAvailable();
        lets.assume().identity("naresh@jains.com", "naresh@india.com", "naresh@indian.com").isAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        lets.assertThat(suggestions).are("naresh@jains.com", "naresh@india.com", "naresh@indian.com");
    }

    @Test
    public void skipAllIdsIfIdsWithAnyCombinationIsNotAvailable() {
        lets.assume().identity((String) anyObject()).isNotAvailable();
        List<String> suggestions = suggester.optionsFor(naresh_from_mumbai);
        assertTrue(suggestions.isEmpty());
    }
}
