package com.agilefaqs.rft.suggestions;

import static com.agilefaqs.rft.suggestions.Email.create;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;

import org.junit.Test;

import com.agilefaqs.rft.domain.DomainNameService;

public class EmailTest {
    private final String firstName = "first_name";
    private final String lastName = "last_name";
    private final DomainNameService dns = mock(DomainNameService.class);
    private final Context lets = new Context(dns);

    @Test
    public void generateIdBasedOnUserNameAndDomainName() {
        lets.assume(firstName, lastName).isNotACelebrityName();
        lets.assume(firstName).isNotARestrictedUserName();
        lets.assume(lastName).isNotARestrictedDomainName();

        String identity = create(firstName, lastName, dns);

        lets.assertThat(identity).isMadeUpOf(firstName, lastName);
    }

    @Test
    public void cannotGenerateIdIfAnyTokenIsNullOrEmpty() {
        assertEmptyString(create(null, lastName, dns));
        assertEmptyString(create(firstName, null, dns));
        assertEmptyString(create(null, null, dns));
        assertEmptyString(create("", lastName, dns));
        assertEmptyString(create(firstName, "", dns));
        assertEmptyString(create("", "", dns));
    }

    @Test
    public void trim1CharAtATimeFromUserNameIfItsReserved() {
        lets.assume(firstName, lastName).isNotACelebrityName();
        lets.assume(firstName).isARestrictedUserName();
        lets.assume("first_nam").isNotARestrictedUserName();
        lets.assume(lastName).isNotARestrictedDomainName();
        lets.assume("first_nam", lastName).isNotACelebrityName();

        String identity = create(firstName, lastName, dns);

        lets.assertThat(identity).isMadeUpOf("first_nam", lastName);

    }

    public void giveUpAfterTrimming5TimesFromReservedUserName() {
        lets.assume(firstName, lastName).isNotACelebrityName();
        lets.assume(firstName, "first_nam", "first_na", "first_n", "first_").areRestrictedUserNames();

        assertEmptyString(create(firstName, lastName, dns));
    }

    @Test
    public void trim1CharAtATimeFromDomainNameIfItsReserved() {
        lets.assume(firstName, lastName).isNotACelebrityName();
        lets.assume(firstName).isNotARestrictedUserName();
        lets.assume(lastName).isARestrictedDomainName();
        lets.assume("last_nam").isNotARestrictedDomainName();
        lets.assume(firstName, "last_nam").isNotACelebrityName();

        String identity = create(firstName, lastName, dns);

        lets.assertThat(identity).isMadeUpOf(firstName, "last_nam");
    }

    public void giveUpAfterTrimming5TimesFromReservedDomainName() {
        lets.assume(firstName, lastName).isNotACelebrityName();
        lets.assume(firstName).isNotARestrictedUserName();
        lets.assume(lastName, "last_nam", "last_na", "last_n", "last_").areRestrictedDomainNames();

        assertEmptyString(create(firstName, lastName, dns));
    }

    @Test
    public void trim1CharAtATimeFromUserNameAndDomainNameIfBothAreReserved() {
        lets.assume(firstName, lastName).isNotACelebrityName();
        lets.assume(firstName, "first_nam", "first_na").areRestrictedUserNames();
        lets.assume("first_n").isNotARestrictedUserName();
        lets.assume(lastName, "last_nam", "last_na").areRestrictedDomainNames();
        lets.assume("last_n").isNotARestrictedDomainName();
        lets.assume("first_n", "last_n").isNotACelebrityName();

        String identity = create(firstName, lastName, dns);

        lets.assertThat(identity).isMadeUpOf("first_n", "last_n");
    }

    @Test
    public void trim1CharAtATimeFromUserNameAndDomainNameIfComboIsCelebrityName() {
        lets.assume(firstName, lastName).isACelebrityName();
        lets.assume("first_nam").isNotARestrictedUserName();
        lets.assume("last_nam").isNotARestrictedDomainName();
        lets.assume("first_nam", "last_nam").isNotACelebrityName();

        String identity = create(firstName, lastName, dns);

        lets.assertThat(identity).isMadeUpOf("first_nam", "last_nam");
    }

    public void giveUpAfterTrimming5TimesUserNameAndDomainNameIfComboIsCelebrityName() {
        lets.assume(firstName, lastName).isACelebrityName();
        lets.assume("first_nam", "last_nam").isACelebrityName();
        lets.assume("first_na", "last_na").isACelebrityName();
        lets.assume("first_n", "last_n").isACelebrityName();
        lets.assume("first_", "last_").isACelebrityName();

        assertEmptyString(create(firstName, lastName, dns));
    }

    private void assertEmptyString(final String value) {
        assertEquals("", value);
    }

}
